@@ -33,14 +33,16 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
-#include <gmodule.h>
 #include <string.h>
-
+#include <libpeas/peas-extension-base.h>
+#include <libpeas/peas-object-module.h>
+#include <libpeas/peas-activatable.h>
 #include <unistd.h>
 #include <lirc/lirc_client.h>
 
 #include "totem-plugin.h"
 #include "totem.h"
+#include "totem-dirs.h"
 
 #define TOTEM_TYPE_LIRC_PLUGIN		(totem_lirc_plugin_get_type ())
 #define TOTEM_LIRC_PLUGIN(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), TOTEM_TYPE_LIRC_PLUGIN, TotemLircPlugin))
@@ -49,20 +51,12 @@
 #define TOTEM_IS_LIRC_PLUGIN_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), TOTEM_TYPE_LIRC_PLUGIN))
 #define TOTEM_LIRC_PLUGIN_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), TOTEM_TYPE_LIRC_PLUGIN, TotemLircPluginClass))
 
-typedef struct
-{
-	TotemPlugin   parent;
-
+typedef struct {
 	GIOChannel *lirc_channel;
 	struct lirc_config *lirc_config;
 
 	TotemObject *totem;
-} TotemLircPlugin;
-
-typedef struct
-{
-	TotemPluginClass parent_class;
-} TotemLircPluginClass;
+} TotemLircPluginPrivate;
 
 /* strings that we recognize as commands from lirc */
 #define TOTEM_IR_COMMAND_PLAY "play"
@@ -94,38 +88,7 @@ typedef struct
 #define TOTEM_IR_SETTING_TOGGLE_REPEAT "setting_repeat"
 #define TOTEM_IR_SETTING_TOGGLE_SHUFFLE "setting_shuffle"
 
-G_MODULE_EXPORT GType register_totem_plugin	(GTypeModule *module);
-GType	totem_lirc_plugin_get_type		(void) G_GNUC_CONST;
-
-static void totem_lirc_plugin_init		(TotemLircPlugin *plugin);
-static void totem_lirc_plugin_finalize		(GObject *object);
-static gboolean impl_activate			(TotemPlugin *plugin, TotemObject *totem, GError **error);
-static void impl_deactivate			(TotemPlugin *plugin, TotemObject *totem);
-
-TOTEM_PLUGIN_REGISTER(TotemLircPlugin, totem_lirc_plugin)
-
-static void
-totem_lirc_plugin_class_init (TotemLircPluginClass *klass)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	TotemPluginClass *plugin_class = TOTEM_PLUGIN_CLASS (klass);
-
-	object_class->finalize = totem_lirc_plugin_finalize;
-
-	plugin_class->activate = impl_activate;
-	plugin_class->deactivate = impl_deactivate;
-}
-
-static void
-totem_lirc_plugin_init (TotemLircPlugin *plugin)
-{
-}
-
-static void
-totem_lirc_plugin_finalize (GObject *object)
-{
-	G_OBJECT_CLASS (totem_lirc_plugin_parent_class)->finalize (object);
-}
+TOTEM_PLUGIN_REGISTER(TOTEM_TYPE_LIRC_PLUGIN, TotemLircPlugin, totem_lirc_plugin)
 
 static char *
 totem_lirc_get_url (const char *str)
@@ -140,7 +103,7 @@ totem_lirc_get_url (const char *str)
 	return g_strdup (s + 1);
 }
 
-static TotemRemoteSetting
+static gint
 totem_lirc_to_setting (const gchar *str, char **url)
 {
 	if (strcmp (str, TOTEM_IR_SETTING_TOGGLE_REPEAT) == 0)
@@ -230,7 +193,7 @@ totem_lirc_read_code (GIOChannel *source, GIOCondition condition, TotemLircPlugi
 	}
 
 	do {
-		ok = lirc_code2char (pi->lirc_config, code, &str);
+		ok = lirc_code2char (pi->priv->lirc_config, code, &str);
 
 		if (ok != 0) {
 			/* Couldn't convert lirc code to string. */
@@ -243,18 +206,16 @@ totem_lirc_read_code (GIOChannel *source, GIOCondition condition, TotemLircPlugi
 		}
 
 		if (g_str_has_prefix (str, TOTEM_IR_SETTING) != FALSE) {
-			TotemRemoteSetting setting;
-
-			setting = totem_lirc_to_setting (str, &url);
+			gint setting = totem_lirc_to_setting (str, &url);
 			if (setting >= 0) {
 				gboolean value;
 
-				value = totem_action_remote_get_setting (pi->totem, setting);
-				totem_action_remote_set_setting (pi->totem, setting, !value);
+				value = totem_action_remote_get_setting (pi->priv->totem, setting);
+				totem_action_remote_set_setting (pi->priv->totem, setting, !value);
 			}
 		} else {
 			cmd = totem_lirc_to_command (str, &url);
-			totem_action_remote (pi->totem, cmd, url);
+			totem_action_remote (pi->priv->totem, cmd, url);
 		}
 		g_free (url);
 	} while (TRUE);
@@ -264,73 +225,75 @@ totem_lirc_read_code (GIOChannel *source, GIOCondition condition, TotemLircPlugi
 	return TRUE;
 }
 
-static gboolean
-impl_activate (TotemPlugin *plugin,
-	       TotemObject *totem,
-	       GError **error)
+static void
+impl_activate (PeasActivatable *plugin)
 {
 	TotemLircPlugin *pi = TOTEM_LIRC_PLUGIN (plugin);
 	char *path;
 	int fd;
 
-	pi->totem = g_object_ref (totem);
+	pi->priv->totem = g_object_ref (g_object_get_data (G_OBJECT (plugin), "object"));
 
-	fd = lirc_init ("Totem", 0);
+	fd = lirc_init ((char*) "Totem", 0);
 	if (fd < 0) {
+		//FIXME
+#if 0
 		g_set_error_literal (error, TOTEM_PLUGIN_ERROR, TOTEM_PLUGIN_ERROR_ACTIVATION,
                                      _("Couldn't initialize lirc."));
 		return FALSE;
+#endif
 	}
 
 	/* Load the default Totem setup */
-	path = totem_plugin_find_file (plugin, "totem_lirc_default");
-	if (path == NULL || lirc_readconfig (path, &pi->lirc_config, NULL) == -1) {
+	path = totem_plugin_find_file ("lirc", "totem_lirc_default");
+	if (path == NULL || lirc_readconfig (path, &pi->priv->lirc_config, NULL) == -1) {
 		g_free (path);
+		//FIXME
+#if 0
 		g_set_error_literal (error, TOTEM_PLUGIN_ERROR, TOTEM_PLUGIN_ERROR_ACTIVATION,
                                      _("Couldn't read lirc configuration."));
+#endif
 		close (fd);
-		return FALSE;
+		return;
 	}
 	g_free (path);
 
 	/* Load the user config, doesn't matter if it's not there */
-	lirc_readconfig (NULL, &pi->lirc_config, NULL);
+	lirc_readconfig (NULL, &pi->priv->lirc_config, NULL);
 
-	pi->lirc_channel = g_io_channel_unix_new (fd);
-	g_io_channel_set_encoding (pi->lirc_channel, NULL, NULL);
-	g_io_channel_set_buffered (pi->lirc_channel, FALSE);
-	g_io_add_watch (pi->lirc_channel, G_IO_IN | G_IO_ERR | G_IO_HUP,
+	pi->priv->lirc_channel = g_io_channel_unix_new (fd);
+	g_io_channel_set_encoding (pi->priv->lirc_channel, NULL, NULL);
+	g_io_channel_set_buffered (pi->priv->lirc_channel, FALSE);
+	g_io_add_watch (pi->priv->lirc_channel, G_IO_IN | G_IO_ERR | G_IO_HUP,
 			(GIOFunc) totem_lirc_read_code, pi);
-
-	return TRUE;
 }
 
 static void
-impl_deactivate	(TotemPlugin *plugin,
-		 TotemObject *totem)
+impl_deactivate (PeasActivatable *plugin)
 {
 	TotemLircPlugin *pi = TOTEM_LIRC_PLUGIN (plugin);
 	GError *error = NULL;
 
-	if (pi->lirc_channel) {
-		g_io_channel_shutdown (pi->lirc_channel, FALSE, &error);
+	if (pi->priv->lirc_channel) {
+		g_io_channel_shutdown (pi->priv->lirc_channel, FALSE, &error);
 		if (error != NULL) {
 			g_warning ("Couldn't destroy lirc connection: %s",
 				   error->message);
 			g_error_free (error);
 		}
-		pi->lirc_channel = NULL;
+		pi->priv->lirc_channel = NULL;
 	}
 
-	if (pi->lirc_config) {
-		lirc_freeconfig (pi->lirc_config);
-		pi->lirc_config = NULL;
+	if (pi->priv->lirc_config) {
+		lirc_freeconfig (pi->priv->lirc_config);
+		pi->priv->lirc_config = NULL;
 
 		lirc_deinit ();
 	}
 
-	if (pi->totem) {
-		g_object_unref (pi->totem);
-		pi->totem = NULL;
+	if (pi->priv->totem) {
+		g_object_unref (pi->priv->totem);
+		pi->priv->totem = NULL;
 	}
 }
+
