@@ -781,6 +781,7 @@ add_volume_to_menu (GVolume *volume,
 	char *menu_item_path;
 
 	disabled = FALSE;
+	device_path = NULL;
 
 	/* Add devices with blank CDs and audio CDs in them, but disable them */
 	if (drive != NULL) {
@@ -1206,8 +1207,7 @@ totem_plugins_response_cb (GtkDialog *dialog,
 			      int response_id,
 			      gpointer data)
 {
-	if (response_id == GTK_RESPONSE_CLOSE)
-		gtk_widget_hide (GTK_WIDGET (dialog));
+	gtk_widget_hide (GTK_WIDGET (dialog));
 }
 
 
@@ -1235,10 +1235,11 @@ plugins_action_callback (GtkAction *action, Totem *totem)
 					 G_CALLBACK (totem_plugins_response_cb),
 					 NULL, 0);
 
-		manager = peas_gtk_plugin_manager_new ();
+		manager = peas_gtk_plugin_manager_new (NULL);
 		gtk_widget_show_all (GTK_WIDGET (manager));
-		gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (totem->plugins))),
-				   manager);
+		gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (totem->plugins))),
+				    manager, TRUE, TRUE, 0);
+		gtk_window_set_default_size (GTK_WINDOW (totem->plugins), 600, 400);
 	}
 
 	gtk_window_present (GTK_WINDOW (totem->plugins));
